@@ -3,7 +3,7 @@ use criterion::{criterion_group, criterion_main, Criterion};
 use scc::ebr::Barrier;
 
 fn barrier_single(c: &mut Criterion) {
-    c.bench_function("barrier", |b| {
+    c.bench_function("EBR: barrier", |b| {
         b.iter(|| {
             let _barrier = Barrier::new();
         })
@@ -12,7 +12,7 @@ fn barrier_single(c: &mut Criterion) {
 
 fn barrier_superposed(c: &mut Criterion) {
     let _barrier = Barrier::new();
-    c.bench_function("superposed barrier", |b| {
+    c.bench_function("EBR: superposed barrier", |b| {
         b.iter(|| {
             let _barrier = Barrier::new();
         })
