@@ -13,7 +13,7 @@ use std::hash::{BuildHasher, Hash, Hasher};
 use std::sync::atomic::AtomicU8;
 use std::sync::atomic::Ordering::{Acquire, Relaxed, Release};
 
-/// `HashTable` defines common functions for `HashIndex`, `HashMap`, and `HashSet`.
+/// `HashTable` defines common functions for hash table implementations.
 pub(super) trait HashTable<K, V, H, const LOCK_FREE: bool>
 where
     K: 'static + Eq + Hash + Sync,
@@ -33,20 +33,19 @@ where
         K: Borrow<Q>,
         Q: Hash + ?Sized,
     {
-        // Generates a hash value.
         let mut h = self.hasher().build_hasher();
         key.hash(&mut h);
         let hash = h.finish();
         (hash, (hash & ((1 << 8) - 1)).try_into().unwrap())
     }
 
-    /// Returns a reference to its build hasher.
+    /// Returns a reference to its [`BuildHasher`].
     fn hasher(&self) -> &H;
 
     /// Copying function.
     fn copier(key: &K, val: &V) -> Option<(K, V)>;
 
-    /// Returns a reference to the `CellArray` pointer.
+    /// Returns a reference to the [`CellArray`] pointer.
     fn cell_array(&self) -> &AtomicArc<CellArray<K, V, LOCK_FREE>>;
 
     /// Returns the minimum allowed capacity.
@@ -80,9 +79,18 @@ where
     }
 
     /// Estimates the number of entries using the given number of cells.
-    fn estimate(array_ref: &CellArray<K, V, LOCK_FREE>, num_cells_to_sample: usize) -> usize {
+    fn estimate(
+        array_ref: &CellArray<K, V, LOCK_FREE>,
+        sampling_index: usize,
+        num_cells_to_sample: usize,
+    ) -> usize {
         let mut num_entries = 0;
-        for i in 0..num_cells_to_sample {
+        let start = if sampling_index + num_cells_to_sample >= array_ref.num_cells() {
+            0
+        } else {
+            sampling_index
+        };
+        for i in start..(start + num_cells_to_sample) {
             num_entries += array_ref.cell(i).num_entries();
         }
         num_entries * (array_ref.num_cells() / num_cells_to_sample)
@@ -90,50 +98,68 @@ where
 
     /// Inserts an entry into the [`HashTable`].
     #[inline]
-    fn insert_entry(&self, key: K, val: V) -> Result<(), (K, V)> {
-        let (hash, partial_hash) = self.hash(&key);
-        let barrier = Barrier::new();
-        let (_, locker, iterator) = self.acquire(&key, hash, partial_hash, &barrier);
-        if iterator.is_some() {
-            return Err((key, val));
+    fn insert_entry<const TRY_LOCK: bool>(
+        &self,
+        key: K,
+        val: V,
+        hash: u64,
+        partial_hash: u8,
+        barrier: &Barrier,
+    ) -> Result<Option<(K, V)>, (K, V)> {
+        match self.acquire::<_, TRY_LOCK>(&key, hash, partial_hash, barrier) {
+            Ok((_, locker, iterator)) => {
+                if iterator.is_some() {
+                    return Ok(Some((key, val)));
+                }
+                locker.insert(key, val, partial_hash, barrier);
+                Ok(None)
+            }
+            Err(_) => Err((key, val)),
         }
-        locker.insert(key, val, partial_hash, &barrier);
-        Ok(())
     }
 
     /// Reads an entry from the [`HashTable`].
     #[inline]
-    fn read_entry<'b, Q, R, F: FnOnce(&'b K, &'b V) -> R>(
+    fn read_entry<'b, Q, R, F: FnMut(&'b K, &'b V) -> R, const TRY_LOCK: bool>(
         &self,
         key_ref: &Q,
-        reader: F,
+        hash: u64,
+        partial_hash: u8,
+        reader: &mut F,
         barrier: &'b Barrier,
-    ) -> Option<R>
+    ) -> Result<Option<R>, ()>
     where
         K: Borrow<Q>,
         Q: Eq + Hash + ?Sized,
     {
-        let (hash, partial_hash) = self.hash(key_ref);
-
         // An acquire fence is required to correctly load the contents of the array.
         let mut current_array_ptr = self.cell_array().load(Acquire, barrier);
         while let Some(current_array_ref) = current_array_ptr.as_ref() {
             if let Some(old_array_ref) = current_array_ref.old_array(barrier).as_ref() {
-                if current_array_ref.partial_rehash(|key| self.hash(key), &Self::copier, barrier) {
-                    current_array_ptr = self.cell_array().load(Acquire, barrier);
-                    continue;
-                }
-                let cell_index = old_array_ref.calculate_cell_index(hash);
-                if LOCK_FREE {
-                    let cell_ref = old_array_ref.cell(cell_index);
-                    if let Some(entry) = cell_ref.search(key_ref, partial_hash, barrier) {
-                        return Some(reader(&entry.0, &entry.1));
-                    }
-                } else if let Some(locker) = Reader::lock(old_array_ref.cell(cell_index), barrier) {
-                    if let Some((key, value)) =
-                        locker.cell_ref().search(key_ref, partial_hash, barrier)
-                    {
-                        return Some(reader(key, value));
+                if !current_array_ref.partial_rehash::<Q, _, _, TRY_LOCK>(
+                    |key| self.hash(key),
+                    &Self::copier,
+                    barrier,
+                ) {
+                    let cell_index = old_array_ref.calculate_cell_index(hash);
+                    if LOCK_FREE {
+                        let cell_ref = old_array_ref.cell(cell_index);
+                        if let Some(entry) = cell_ref.search(key_ref, partial_hash, barrier) {
+                            return Ok(Some(reader(&entry.0, &entry.1)));
+                        }
+                    } else {
+                        let lock_result = if TRY_LOCK {
+                            Reader::try_lock(old_array_ref.cell(cell_index), barrier)?
+                        } else {
+                            Reader::lock(old_array_ref.cell(cell_index), barrier)
+                        };
+                        if let Some(locker) = lock_result {
+                            if let Some((key, value)) =
+                                locker.cell().search(key_ref, partial_hash, barrier)
+                            {
+                                return Ok(Some(reader(key, value)));
+                            }
+                        }
                     }
                 }
             }
@@ -141,12 +167,19 @@ where
             if LOCK_FREE {
                 let cell_ref = current_array_ref.cell(cell_index);
                 if let Some(entry) = cell_ref.search(key_ref, partial_hash, barrier) {
-                    return Some(reader(&entry.0, &entry.1));
+                    return Ok(Some(reader(&entry.0, &entry.1)));
                 }
-            } else if let Some(locker) = Reader::lock(current_array_ref.cell(cell_index), barrier) {
-                if let Some((key, value)) = locker.cell_ref().search(key_ref, partial_hash, barrier)
-                {
-                    return Some(reader(key, value));
+            } else {
+                let lock_result = if TRY_LOCK {
+                    Reader::try_lock(current_array_ref.cell(cell_index), barrier)?
+                } else {
+                    Reader::lock(current_array_ref.cell(cell_index), barrier)
+                };
+                if let Some(locker) = lock_result {
+                    if let Some((key, value)) = locker.cell().search(key_ref, partial_hash, barrier)
+                    {
+                        return Ok(Some(reader(key, value)));
+                    }
                 }
             }
             let new_current_array_ptr = self.cell_array().load(Acquire, barrier);
@@ -156,23 +189,25 @@ where
             // The pointer value has changed.
             current_array_ptr = new_current_array_ptr;
         }
-        None
+        Ok(None)
     }
 
     /// Removes an entry if the condition is met.
     #[inline]
-    fn remove_entry<Q, F: FnOnce(&V) -> bool>(
+    fn remove_entry<Q, F: FnMut(&V) -> bool, const TRY_LOCK: bool>(
         &self,
         key_ref: &Q,
-        condition: F,
-    ) -> (Option<(K, V)>, bool)
+        hash: u64,
+        partial_hash: u8,
+        condition: &mut F,
+        barrier: &Barrier,
+    ) -> Result<(Option<(K, V)>, bool), ()>
     where
         K: Borrow<Q>,
         Q: Eq + Hash + ?Sized,
     {
-        let (hash, partial_hash) = self.hash(key_ref);
-        let barrier = Barrier::new();
-        let (cell_index, locker, iterator) = self.acquire(key_ref, hash, partial_hash, &barrier);
+        let (cell_index, locker, iterator) =
+            self.acquire::<Q, TRY_LOCK>(key_ref, hash, partial_hash, barrier)?;
         if let Some(mut iterator) = iterator {
             let remove = if let Some((_, v)) = iterator.get() {
                 condition(v)
@@ -181,40 +216,44 @@ where
             };
             if remove {
                 let result = locker.erase(&mut iterator);
-                if cell_index < ARRAY_SIZE && locker.cell_ref().num_entries() < ARRAY_SIZE / 16 {
+                if (cell_index % ARRAY_SIZE) == 0 && locker.cell().num_entries() < ARRAY_SIZE / 16 {
                     drop(locker);
                     if let Some(current_array_ref) =
-                        self.cell_array().load(Acquire, &barrier).as_ref()
+                        self.cell_array().load(Acquire, barrier).as_ref()
                     {
-                        if current_array_ref.old_array(&barrier).is_null()
+                        if current_array_ref.old_array(barrier).is_null()
                             && current_array_ref.num_entries() > self.minimum_capacity()
                         {
-                            self.try_shrink(current_array_ref, cell_index, &barrier);
+                            self.try_shrink(current_array_ref, cell_index, barrier);
                         }
                     }
                 }
-                return (result, true);
+                return Ok((result, true));
             }
         }
-        (None, false)
+        Ok((None, false))
     }
 
     /// Acquires a [`Locker`] and [`EntryIterator`].
     ///
-    /// In case it successfully found the key, it returns a [`EntryIterator`]. Not returning a
-    /// [`EntryIterator`] means that the key does not exist.
+    /// It returns an error if locking failed, otherwise it returns an [`EntryIterator`] if the
+    /// key exists or `None` if not.
     #[inline]
-    fn acquire<'h, 'b, Q>(
+    #[allow(clippy::type_complexity)]
+    fn acquire<'h, 'b, Q, const TRY_LOCK: bool>(
         &'h self,
         key_ref: &Q,
         hash: u64,
         partial_hash: u8,
         barrier: &'b Barrier,
-    ) -> (
-        usize,
-        Locker<'b, K, V, LOCK_FREE>,
-        Option<EntryIterator<'b, K, V, LOCK_FREE>>,
-    )
+    ) -> Result<
+        (
+            usize,
+            Locker<'b, K, V, LOCK_FREE>,
+            Option<EntryIterator<'b, K, V, LOCK_FREE>>,
+        ),
+        (),
+    >
     where
         K: Borrow<Q>,
         Q: Hash + Eq + ?Sized,
@@ -238,42 +277,55 @@ where
             let current_array_ptr = self.cell_array().load(Acquire, barrier);
             let current_array_ref = current_array_ptr.as_ref().unwrap();
             if let Some(old_array_ref) = current_array_ref.old_array(barrier).as_ref() {
-                if current_array_ref.partial_rehash(|key| self.hash(key), &Self::copier, barrier) {
-                    continue;
-                }
-                check_resize = false;
-                let cell_index = old_array_ref.calculate_cell_index(hash);
-                if let Some(mut locker) = Locker::lock(old_array_ref.cell(cell_index), barrier) {
-                    if let Some(iterator) = locker.cell_ref().get(key_ref, partial_hash, barrier) {
-                        return (cell_index, locker, Some(iterator));
+                if !current_array_ref.partial_rehash::<Q, _, _, TRY_LOCK>(
+                    |key| self.hash(key),
+                    &Self::copier,
+                    barrier,
+                ) {
+                    check_resize = false;
+                    let cell_index = old_array_ref.calculate_cell_index(hash);
+                    let lock_result = if TRY_LOCK {
+                        Locker::try_lock(old_array_ref.cell(cell_index), barrier)?
+                    } else {
+                        Locker::lock(old_array_ref.cell(cell_index), barrier)
+                    };
+                    if let Some(mut locker) = lock_result {
+                        if let Some(iterator) = locker.cell().get(key_ref, partial_hash, barrier) {
+                            return Ok((cell_index, locker, Some(iterator)));
+                        }
+                        // Kills the Cell.
+                        current_array_ref.kill_cell::<Q, _, _, TRY_LOCK>(
+                            &mut locker,
+                            old_array_ref,
+                            cell_index,
+                            &|key| self.hash(key),
+                            &Self::copier,
+                            barrier,
+                        )?;
                     }
-                    // Kills the Cell.
-                    current_array_ref.kill_cell(
-                        &mut locker,
-                        old_array_ref,
-                        cell_index,
-                        &|key| self.hash(key),
-                        &Self::copier,
-                        barrier,
-                    );
                 }
             }
             let cell_index = current_array_ref.calculate_cell_index(hash);
 
-            // Tries to resize the array.
+            // Try to resize the array.
             let num_entries = current_array_ref.cell(cell_index).num_entries();
-            if check_resize && num_entries >= ARRAY_SIZE {
-                // Triggers resize if the estimated load factor is greater than 7/8.
+            if cell_index % ARRAY_SIZE == 0 && check_resize && num_entries >= ARRAY_SIZE {
+                // Trigger resize if the estimated load factor is greater than 7/8.
                 check_resize = false;
                 self.try_enlarge(current_array_ref, cell_index, num_entries, barrier);
                 continue;
             }
 
-            if let Some(locker) = Locker::lock(current_array_ref.cell(cell_index), barrier) {
-                if let Some(iterator) = locker.cell_ref().get(key_ref, partial_hash, barrier) {
-                    return (cell_index, locker, Some(iterator));
+            let lock_result = if TRY_LOCK {
+                Locker::try_lock(current_array_ref.cell(cell_index), barrier)?
+            } else {
+                Locker::lock(current_array_ref.cell(cell_index), barrier)
+            };
+            if let Some(locker) = lock_result {
+                if let Some(iterator) = locker.cell().get(key_ref, partial_hash, barrier) {
+                    return Ok((cell_index, locker, Some(iterator)));
                 }
-                return (cell_index, locker, None);
+                return Ok((cell_index, locker, None));
             }
 
             // Reaching here means that `self.array` is updated.
@@ -353,6 +405,7 @@ where
             }
         }
 
+        let mut sampling_index = 0;
         let mut resize = true;
         while resize {
             let _mutex_guard = scopeguard::guard(&mut resize, |resize| {
@@ -371,7 +424,9 @@ where
             let capacity = current_array_ref.num_entries();
             let num_cells = current_array_ref.num_cells();
             let num_cells_to_sample = (num_cells / 8).max(2).min(4096);
-            let estimated_num_entries = Self::estimate(current_array_ref, num_cells_to_sample);
+            let estimated_num_entries =
+                Self::estimate(current_array_ref, sampling_index, num_cells_to_sample);
+            sampling_index = sampling_index.wrapping_add(num_cells_to_sample);
             let new_capacity = if estimated_num_entries >= (capacity / 8) * 7 {
                 let max_capacity = 1_usize << (std::mem::size_of::<usize>() * 8 - 1);
                 if capacity == max_capacity {
