@@ -7,14 +7,6 @@
 //! * [`HashIndex`](HashIndex).
 //! * [`TreeIndex`](TreeIndex).
 //!
-//! # [Awaitable concurrent containers](awaitable)
-//!
-//! The concurrent container types in `awaitable` are meant for asynchronous code blocks as their
-//! key operations are implemented in asynchronous methods where execution can be suspended on a
-//! conflict over resources among threads.
-//!
-//! * [`HashMap`](awaitable::HashMap).
-//!
 //! # [`EBR`](ebr)
 //!
 //! The [`ebr`] module implements epoch-based reclamation for every container type in this crate.
@@ -34,8 +26,8 @@ pub use hash_set::HashSet;
 pub mod tree_index;
 pub use tree_index::TreeIndex;
 
-pub mod awaitable;
 pub mod ebr;
 
+mod async_yield;
 mod hash_table;
 mod tests;
