@@ -1,10 +1,8 @@
 #[cfg(test)]
 mod hashmap_test {
-    use crate::HashMap;
+    use crate::ebr;
+    use crate::{HashIndex, HashMap};
 
-    use proptest::prelude::*;
-    use proptest::strategy::{Strategy, ValueTree};
-    use proptest::test_runner::TestRunner;
     use std::collections::BTreeSet;
     use std::hash::{Hash, Hasher};
     use std::sync::atomic::Ordering::{Acquire, Relaxed, Release};
@@ -12,8 +10,180 @@ mod hashmap_test {
     use std::sync::{Arc, Barrier};
     use std::thread;
 
+    use proptest::prelude::*;
+    use proptest::strategy::{Strategy, ValueTree};
+    use proptest::test_runner::TestRunner;
+    use tokio::sync::Barrier as AsyncBarrier;
+
+    struct R(&'static AtomicUsize);
+    impl R {
+        fn new(cnt: &'static AtomicUsize) -> R {
+            cnt.fetch_add(1, Relaxed);
+            R(cnt)
+        }
+    }
+    impl Clone for R {
+        fn clone(&self) -> Self {
+            self.0.fetch_add(1, Relaxed);
+            R(self.0)
+        }
+    }
+    impl Drop for R {
+        fn drop(&mut self) {
+            self.0.fetch_sub(1, Relaxed);
+        }
+    }
+
+    #[tokio::test]
+    async fn hashmap_insert_drop() {
+        static CNT: AtomicUsize = AtomicUsize::new(0);
+        let hashmap: HashMap<usize, R> = HashMap::default();
+
+        let workload_size = 1024;
+        for k in 0..workload_size {
+            assert!(hashmap.insert_async(k, R::new(&CNT)).await.is_ok());
+        }
+        assert_eq!(CNT.load(Relaxed), workload_size);
+        assert_eq!(hashmap.len(), workload_size);
+        drop(hashmap);
+
+        while CNT.load(Relaxed) != 0 {
+            drop(ebr::Barrier::new());
+            tokio::task::yield_now().await;
+        }
+    }
+
+    #[tokio::test]
+    async fn hashmap_clear() {
+        static CNT: AtomicUsize = AtomicUsize::new(0);
+        let hashmap: HashMap<usize, R> = HashMap::default();
+
+        let workload_size = 1024;
+
+        for _ in 0..2 {
+            for k in 0..workload_size {
+                assert!(hashmap.insert_async(k, R::new(&CNT)).await.is_ok());
+            }
+            assert_eq!(CNT.load(Relaxed), workload_size);
+            assert_eq!(hashmap.len(), workload_size);
+            assert_eq!(hashmap.clear_async().await, workload_size);
+        }
+
+        while CNT.load(Relaxed) != 0 {
+            drop(ebr::Barrier::new());
+            tokio::task::yield_now().await;
+        }
+    }
+
+    #[tokio::test(flavor = "multi_thread", worker_threads = 16)]
+    async fn hashmap_integer_key() {
+        let hashmap: Arc<HashMap<usize, usize>> = Arc::new(HashMap::default());
+
+        let num_tasks = 8;
+        let workload_size = 256;
+        let mut task_handles = Vec::with_capacity(num_tasks);
+        let barrier = Arc::new(AsyncBarrier::new(num_tasks));
+        for task_id in 0..num_tasks {
+            let barrier_cloned = barrier.clone();
+            let hashmap_cloned = hashmap.clone();
+            task_handles.push(tokio::task::spawn(async move {
+                barrier_cloned.wait().await;
+                let range = (task_id * workload_size)..((task_id + 1) * workload_size);
+                for id in range.clone() {
+                    let result = hashmap_cloned.update_async(&id, |_, _| 1).await;
+                    assert!(result.is_none());
+                }
+                for id in range.clone() {
+                    if id % 5 == 0 {
+                        hashmap_cloned.upsert_async(id, || id, |_, v| *v = id).await;
+                    } else {
+                        let result = hashmap_cloned.insert_async(id, id).await;
+                        assert!(result.is_ok());
+                    }
+                }
+                for id in range.clone() {
+                    if id % 7 == 0 {
+                        hashmap_cloned
+                            .upsert_async(id, || id, |_, v| *v = id + 1)
+                            .await;
+                    } else {
+                        let result = hashmap_cloned
+                            .update_async(&id, |_, v| {
+                                *v += 1;
+                                *v
+                            })
+                            .await;
+                        assert_eq!(result, Some(id + 1));
+                    }
+                }
+                for id in range.clone() {
+                    let result = hashmap_cloned.read_async(&id, |_, v| *v).await;
+                    assert_eq!(result, Some(id + 1));
+                }
+                for id in range.clone() {
+                    let result = hashmap_cloned.remove_if_async(&id, |v| *v == id + 1).await;
+                    assert_eq!(result, Some((id, id + 1)));
+                }
+                for id in range {
+                    let result = hashmap_cloned.remove_if_async(&id, |v| *v == id + 1).await;
+                    assert_eq!(result, None);
+                }
+            }));
+        }
+
+        for r in futures::future::join_all(task_handles).await {
+            assert!(r.is_ok());
+        }
+
+        assert_eq!(hashmap.len(), 0);
+    }
+
+    #[tokio::test(flavor = "multi_thread", worker_threads = 4)]
+    async fn hashmap_retain_for_each() {
+        let hashmap: Arc<HashMap<usize, usize>> = Arc::new(HashMap::default());
+
+        let num_tasks = 8;
+        let workload_size = 256;
+        let mut task_handles = Vec::with_capacity(num_tasks);
+        let barrier = Arc::new(AsyncBarrier::new(num_tasks));
+        for task_id in 0..num_tasks {
+            let barrier_cloned = barrier.clone();
+            let hashmap_cloned = hashmap.clone();
+            task_handles.push(tokio::task::spawn(async move {
+                barrier_cloned.wait().await;
+                let range = (task_id * workload_size)..((task_id + 1) * workload_size);
+                for id in range.clone() {
+                    let result = hashmap_cloned.insert_async(id, id).await;
+                    assert!(result.is_ok());
+                }
+                for id in range.clone() {
+                    let result = hashmap_cloned.insert_async(id, id).await;
+                    assert_eq!(result, Err((id, id)));
+                }
+                let mut iterated = 0;
+                hashmap_cloned
+                    .for_each_async(|k, _| {
+                        if range.contains(k) {
+                            iterated += 1;
+                        }
+                    })
+                    .await;
+                assert!(iterated >= workload_size);
+
+                let (_, removed) = hashmap_cloned.retain_async(|k, _| !range.contains(k)).await;
+                assert_eq!(removed, workload_size);
+            }));
+        }
+
+        for r in futures::future::join_all(task_handles).await {
+            assert!(r.is_ok());
+        }
+
+        assert_eq!(hashmap.len(), 0);
+    }
+
     #[test]
-    fn string_key() {
+    fn hashmap_string_key() {
         let hashmap1: HashMap<String, u32> = HashMap::default();
         let hashmap2: HashMap<u32, String> = HashMap::default();
         let mut checker1 = BTreeSet::new();
@@ -49,7 +219,7 @@ mod hashmap_test {
     }
 
     #[test]
-    fn accessor() {
+    fn hashmap_accessor() {
         let data_size = 4096;
         for _ in 0..16 {
             let hashmap: Arc<HashMap<u64, u64>> = Arc::new(HashMap::default());
@@ -148,7 +318,7 @@ mod hashmap_test {
 
     proptest! {
         #[test]
-        fn insert(key in 0_usize..16) {
+        fn hashmap_insert(key in 0_usize..16) {
             let range = 4096;
             let checker = Arc::new(AtomicUsize::new(0));
             let hashmap: HashMap<Data, Data> = HashMap::default();
@@ -196,26 +366,39 @@ mod hashmap_test {
             }
             assert_eq!(checker.load(Relaxed), range * 2);
             drop(hashmap);
-            assert_eq!(checker.load(Relaxed), 0);
+
+            while checker.load(Relaxed) != 0 {
+                drop(ebr::Barrier::new());
+                std::thread::yield_now();
+            }
         }
     }
-}
 
-#[cfg(test)]
-mod hashindex_test {
-    use crate::ebr;
-    use crate::HashIndex;
+    #[tokio::test]
+    async fn hashindex_clear() {
+        static CNT: AtomicUsize = AtomicUsize::new(0);
+        let hashindex: HashIndex<usize, R> = HashIndex::default();
 
-    use proptest::strategy::{Strategy, ValueTree};
-    use proptest::test_runner::TestRunner;
-    use std::collections::BTreeSet;
-    use std::sync::atomic::AtomicU64;
-    use std::sync::atomic::Ordering::{Acquire, Release};
-    use std::sync::{Arc, Barrier};
-    use std::thread;
+        let workload_size = 1024;
+
+        for _ in 0..2 {
+            for k in 0..workload_size {
+                assert!(hashindex.insert_async(k, R::new(&CNT)).await.is_ok());
+            }
+            assert!(CNT.load(Relaxed) >= workload_size);
+            assert_eq!(hashindex.len(), workload_size);
+            assert_eq!(hashindex.clear_async().await, workload_size);
+        }
+        drop(hashindex);
+
+        while CNT.load(Relaxed) != 0 {
+            drop(ebr::Barrier::new());
+            tokio::task::yield_now().await;
+        }
+    }
 
     #[test]
-    fn string_key() {
+    fn hashindex_string_key() {
         let hashindex1: HashIndex<String, u32> = HashIndex::default();
         let hashindex2: HashIndex<u32, String> = HashIndex::default();
         let mut checker1 = BTreeSet::new();
@@ -248,7 +431,7 @@ mod hashindex_test {
     }
 
     #[test]
-    fn visitor() {
+    fn hashindex_visitor() {
         let data_size = 4096;
         for _ in 0..64 {
             let hashindex: Arc<HashIndex<u64, u64>> = Arc::new(HashIndex::default());
@@ -297,6 +480,7 @@ mod hashindex_test {
                     barrier.wait();
                 }
                 assert!(hashindex.remove(&i));
+                assert!(hashindex.read(&i, |_, _| ()).is_none());
                 removed.store(i, Release);
             }
             thread_handle.join().unwrap();
@@ -317,6 +501,125 @@ mod treeindex_test {
     use std::sync::{Arc, Barrier};
     use std::thread;
 
+    use tokio::sync::Barrier as AsyncBarrier;
+
+    struct R(&'static AtomicUsize);
+    impl R {
+        fn new(cnt: &'static AtomicUsize) -> R {
+            cnt.fetch_add(1, Relaxed);
+            R(cnt)
+        }
+    }
+    impl Clone for R {
+        fn clone(&self) -> Self {
+            self.0.fetch_add(1, Relaxed);
+            R(self.0)
+        }
+    }
+    impl Drop for R {
+        fn drop(&mut self) {
+            self.0.fetch_sub(1, Relaxed);
+        }
+    }
+
+    #[tokio::test]
+    async fn insert_drop() {
+        static CNT: AtomicUsize = AtomicUsize::new(0);
+        let tree: TreeIndex<usize, R> = TreeIndex::default();
+
+        let workload_size = 1024;
+        for k in 0..workload_size {
+            assert!(tree.insert_async(k, R::new(&CNT)).await.is_ok());
+        }
+        assert!(CNT.load(Relaxed) >= workload_size);
+        assert_eq!(tree.len(), workload_size);
+        drop(tree);
+
+        while CNT.load(Relaxed) != 0 {
+            drop(ebr::Barrier::new());
+            tokio::task::yield_now().await;
+        }
+    }
+
+    #[tokio::test]
+    async fn insert_remove() {
+        static CNT: AtomicUsize = AtomicUsize::new(0);
+        let tree: TreeIndex<usize, R> = TreeIndex::default();
+
+        let workload_size = 1024;
+        for k in 0..workload_size {
+            assert!(tree.insert_async(k, R::new(&CNT)).await.is_ok());
+        }
+        assert!(CNT.load(Relaxed) >= workload_size);
+        assert_eq!(tree.len(), workload_size);
+        for k in 0..workload_size {
+            assert!(tree.remove_async(&k).await);
+        }
+        assert_eq!(tree.len(), 0);
+
+        while CNT.load(Relaxed) != 0 {
+            drop(ebr::Barrier::new());
+            tokio::task::yield_now().await;
+        }
+    }
+
+    #[tokio::test]
+    async fn clear() {
+        static CNT: AtomicUsize = AtomicUsize::new(0);
+        let tree: TreeIndex<usize, R> = TreeIndex::default();
+
+        let workload_size = 1024;
+        for k in 0..workload_size {
+            assert!(tree.insert_async(k, R::new(&CNT)).await.is_ok());
+        }
+        assert!(CNT.load(Relaxed) >= workload_size);
+        assert_eq!(tree.len(), workload_size);
+        tree.clear();
+
+        while CNT.load(Relaxed) != 0 {
+            drop(ebr::Barrier::new());
+            tokio::task::yield_now().await;
+        }
+    }
+
+    #[tokio::test(flavor = "multi_thread", worker_threads = 16)]
+    async fn integer_key() {
+        let num_tasks = 8;
+        let workload_size = 256;
+        for _ in 0..256 {
+            let tree: Arc<TreeIndex<usize, usize>> = Arc::new(TreeIndex::default());
+            let mut task_handles = Vec::with_capacity(num_tasks);
+            let barrier = Arc::new(AsyncBarrier::new(num_tasks));
+            for task_id in 0..num_tasks {
+                let barrier_cloned = barrier.clone();
+                let tree_cloned = tree.clone();
+                task_handles.push(tokio::task::spawn(async move {
+                    barrier_cloned.wait().await;
+                    let range = (task_id * workload_size)..((task_id + 1) * workload_size);
+                    for id in range.clone() {
+                        assert!(tree_cloned.insert_async(id, id).await.is_ok());
+                        assert!(tree_cloned.insert_async(id, id).await.is_err());
+                    }
+                    for id in range.clone() {
+                        let result = tree_cloned.read(&id, |_, v| *v);
+                        assert_eq!(result, Some(id));
+                    }
+                    for id in range.clone() {
+                        assert!(tree_cloned.remove_if_async(&id, |v| *v == id).await);
+                    }
+                    for id in range {
+                        assert!(!tree_cloned.remove_if_async(&id, |v| *v == id).await);
+                    }
+                }));
+            }
+
+            for r in futures::future::join_all(task_handles).await {
+                assert!(r.is_ok());
+            }
+            assert_eq!(tree.len(), 0);
+        }
+    }
+
     #[test]
     fn reclaim() {
         static INST_CNT: AtomicUsize = AtomicUsize::new(0);
@@ -666,149 +969,3 @@ mod treeindex_test {
         }
     }
 }
-
-#[cfg(test)]
-mod hashmap_test_async {
-    use crate::awaitable::HashMap;
-
-    use std::sync::atomic::AtomicUsize;
-    use std::sync::atomic::Ordering::Relaxed;
-    use std::sync::Arc;
-
-    use tokio::sync::Barrier;
-
-    struct R(&'static AtomicUsize);
-    impl R {
-        fn new(cnt: &'static AtomicUsize) -> R {
-            cnt.fetch_add(1, Relaxed);
-            R(cnt)
-        }
-    }
-    impl Drop for R {
-        fn drop(&mut self) {
-            self.0.fetch_sub(1, Relaxed);
-        }
-    }
-
-    #[tokio::test]
-    async fn insert_drop() {
-        static CNT: AtomicUsize = AtomicUsize::new(0);
-        let hashmap: HashMap<usize, R> = HashMap::default();
-
-        let workload_size = 1024;
-        for k in 0..workload_size {
-            assert!(hashmap.insert(k, R::new(&CNT)).await.is_ok());
-        }
-        assert_eq!(CNT.load(Relaxed), workload_size);
-        assert_eq!(hashmap.len(), workload_size);
-        drop(hashmap);
-
-        while CNT.load(Relaxed) != 0 {
-            drop(crate::ebr::Barrier::new());
-            tokio::task::yield_now().await;
-        }
-    }
-
-    #[tokio::test]
-    async fn clear() {
-        static CNT: AtomicUsize = AtomicUsize::new(0);
-        let hashmap: HashMap<usize, R> = HashMap::default();
-
-        let workload_size = 1024;
-        for k in 0..workload_size {
-            assert!(hashmap.insert(k, R::new(&CNT)).await.is_ok());
-        }
-        assert_eq!(CNT.load(Relaxed), workload_size);
-        assert_eq!(hashmap.len(), workload_size);
-        assert_eq!(hashmap.clear().await, workload_size);
-
-        while CNT.load(Relaxed) != 0 {
-            drop(crate::ebr::Barrier::new());
-            tokio::task::yield_now().await;
-        }
-    }
-
-    #[tokio::test(flavor = "multi_thread", worker_threads = 16)]
-    async fn integer_key() {
-        let hashmap: Arc<HashMap<usize, usize>> = Arc::new(HashMap::default());
-
-        let num_tasks = 8;
-        let workload_size = 256;
-        let mut task_handles = Vec::with_capacity(num_tasks);
-        let barrier = Arc::new(Barrier::new(num_tasks));
-        for task_id in 0..num_tasks {
-            let barrier_cloned = barrier.clone();
-            let hashmap_cloned = hashmap.clone();
-            task_handles.push(tokio::task::spawn(async move {
-                barrier_cloned.wait().await;
-                let range = (task_id * workload_size)..((task_id + 1) * workload_size);
-                for id in range.clone() {
-                    let result = hashmap_cloned.insert(id, id).await;
-                    assert!(result.is_ok());
-                }
-                for id in range.clone() {
-                    let result = hashmap_cloned.read(&id, |_, v| *v).await;
-                    assert_eq!(result, Some(id));
-                }
-                for id in range.clone() {
-                    let result = hashmap_cloned.remove_if(&id, |v| *v == id).await;
-                    assert_eq!(result, Some((id, id)));
-                }
-                for id in range {
-                    let result = hashmap_cloned.remove_if(&id, |v| *v == id).await;
-                    assert_eq!(result, None);
-                }
-            }));
-        }
-
-        for r in futures::future::join_all(task_handles).await {
-            assert!(r.is_ok());
-        }
-
-        assert_eq!(hashmap.len(), 0);
-    }
-
-    #[tokio::test(flavor = "multi_thread", worker_threads = 4)]
-    async fn retain_for_each() {
-        let hashmap: Arc<HashMap<usize, usize>> = Arc::new(HashMap::default());
-
-        let num_tasks = 8;
-        let workload_size = 256;
-        let mut task_handles = Vec::with_capacity(num_tasks);
-        let barrier = Arc::new(Barrier::new(num_tasks));
-        for task_id in 0..num_tasks {
-            let barrier_cloned = barrier.clone();
-            let hashmap_cloned = hashmap.clone();
-            task_handles.push(tokio::task::spawn(async move {
-                barrier_cloned.wait().await;
-                let range = (task_id * workload_size)..((task_id + 1) * workload_size);
-                for id in range.clone() {
-                    let result = hashmap_cloned.insert(id, id).await;
-                    assert!(result.is_ok());
-                }
-                for id in range.clone() {
-                    let result = hashmap_cloned.insert(id, id).await;
-                    assert_eq!(result, Err((id, id)));
-                }
-                let mut iterated = 0;
-                hashmap_cloned
-                    .for_each(|k, _| {
-                        if range.contains(k) {
-                            iterated += 1;
-                        }
-                    })
-                    .await;
-                assert!(iterated >= workload_size);
-
-                let (_, removed) = hashmap_cloned.retain(|k, _| !range.contains(k)).await;
-                assert_eq!(removed, workload_size);
-            }));
-        }
-
-        for r in futures::future::join_all(task_handles).await {
-            assert!(r.is_ok());
-        }
-
-        assert_eq!(hashmap.len(), 0);
-    }
-}
