@@ -71,7 +71,7 @@ mod benchmark {
         #[inline(always)]
         fn scan_test(&self) -> usize {
             let mut scanned = 0;
-            self.for_each(|_, _| scanned += 1);
+            self.scan(|_, _| scanned += 1);
             scanned
         }
 
@@ -569,6 +569,7 @@ mod benchmark {
             };
             let (duration, total_num_operations) =
                 perform(num_threads, 0, treeindex.clone(), insert.clone());
+            assert_eq!(treeindex.len(), total_num_operations);
             println!(
                 "treeindex-insert-local: {}, {:?}, {}, depth = {}",
                 num_threads,
@@ -645,6 +646,7 @@ mod benchmark {
             };
             let (duration, total_num_operations) =
                 perform(num_threads, 0, treeindex.clone(), insert.clone());
+            assert_eq!(treeindex.len(), total_num_operations / 2);
             println!(
                 "treeindex-insert-local-remote: {}, {:?}, {}, depth = {}",
                 num_threads,
@@ -715,21 +717,28 @@ mod benchmark {
 
     #[ignore]
     #[test]
-    fn sync_full_scale_benchmarks() {
-        hashmap_benchmark::<usize>(1024 * 1024 * 128, vec![11, 11, 11, 22, 22, 22, 44, 44, 44]);
+    fn full_scale_benchmarks() {
+        hashmap_benchmark::<usize>(
+            1024 * 1024 * 16,
+            vec![1, 1, 1, 4, 4, 4, 16, 16, 16, 64, 64, 64],
+        );
         println!("----");
-        /*
-        hashindex_benchmark::<usize>(1024 * 1024 * 4, vec![11, 11, 11, 22, 22, 22, 44, 44, 44]);
+        hashindex_benchmark::<usize>(
+            1024 * 1024 * 16,
+            vec![1, 1, 1, 4, 4, 4, 16, 16, 16, 64, 64, 64],
+        );
         println!("----");
-        treeindex_benchmark::<usize>(1024 * 1024 * 4, vec![11, 11, 11, 22, 22, 22, 44, 44, 44]);
+        treeindex_benchmark::<usize>(
+            1024 * 1024 * 16,
+            vec![1, 1, 1, 4, 4, 4, 16, 16, 16, 64, 64, 64],
+        );
         println!("----");
-        */
     }
 }
 
 #[cfg(test)]
 mod benchmark_async {
-    use crate::awaitable::HashMap;
+    use crate::HashMap;
 
     use std::collections::hash_map::RandomState;
     use std::sync::atomic::AtomicUsize;
@@ -787,7 +796,9 @@ mod benchmark_async {
                 let per_task_workload_size = workload_cloned.size * per_op_workload_size;
                 barrier_cloned.wait().await;
                 for _ in 0..workload_cloned.scan {
-                    hashmap_cloned.for_each(|_, _| num_operations += 1).await;
+                    hashmap_cloned
+                        .for_each_async(|_, _| num_operations += 1)
+                        .await;
                 }
                 for i in 0..per_task_workload_size {
                     let remote_task_id = if num_tasks < 2 {
@@ -801,7 +812,7 @@ mod benchmark_async {
                             + i * per_op_workload_size
                             + j
                             + start_index;
-                        let result = hashmap_cloned.insert(local_index, i).await;
+                        let result = hashmap_cloned.insert_async(local_index, i).await;
                         assert!(result.is_ok() || workload_cloned.has_remote_op());
                         num_operations += 1;
                     }
@@ -810,7 +821,7 @@ mod benchmark_async {
                             + i * per_op_workload_size
                             + j
                             + start_index;
-                        let _result = hashmap_cloned.insert(remote_index, i).await;
+                        let _result = hashmap_cloned.insert_async(remote_index, i).await;
                         num_operations += 1;
                     }
                     for j in 0..workload_cloned.read_local {
@@ -818,7 +829,7 @@ mod benchmark_async {
                             + i * per_op_workload_size
                             + j
                             + start_index;
-                        let result = hashmap_cloned.read(&local_index, |_, _| ()).await;
+                        let result = hashmap_cloned.read_async(&local_index, |_, _| ()).await;
                         assert!(result.is_some() || workload_cloned.has_remote_op());
                         num_operations += 1;
                     }
@@ -827,7 +838,7 @@ mod benchmark_async {
                             + i * per_op_workload_size
                             + j
                             + start_index;
-                        let _result = hashmap_cloned.read(&remote_index, |_, _| ()).await;
+                        let _result = hashmap_cloned.read_async(&remote_index, |_, _| ()).await;
                         num_operations += 1;
                     }
                     for j in 0..workload_cloned.remove_local {
@@ -835,7 +846,7 @@ mod benchmark_async {
                             + i * per_op_workload_size
                             + j
                             + start_index;
-                        let result = hashmap_cloned.remove(&local_index).await;
+                        let result = hashmap_cloned.remove_async(&local_index).await;
                         assert!(result.is_some() || workload_cloned.has_remote_op());
                         num_operations += 1;
                     }
@@ -844,7 +855,7 @@ mod benchmark_async {
                             + i * per_op_workload_size
                             + j
                             + start_index;
-                        let _result = hashmap_cloned.remove(&remote_index).await;
+                        let _result = hashmap_cloned.remove_async(&remote_index).await;
                         num_operations += 1;
                     }
                 }
