@@ -1,19 +1,18 @@
 //! The module implements [`TreeIndex`].
 
-mod error;
+mod internal_node;
 mod leaf;
 mod leaf_node;
 mod node;
 
-use super::ebr::{Arc, AtomicArc, Barrier, Tag};
+use crate::async_yield;
+use crate::ebr::{Arc, AtomicArc, Barrier, Ptr, Tag};
 
-use error::{InsertError, RemoveError, SearchError};
-use leaf::{Leaf, Scanner};
+use leaf::{InsertResult, Leaf, RemoveResult, Scanner};
 use node::Node;
 
 use std::borrow::Borrow;
 use std::cmp::Ordering;
-use std::fmt;
 use std::iter::FusedIterator;
 use std::ops::Bound::{Excluded, Included, Unbounded};
 use std::ops::RangeBounds;
@@ -21,9 +20,10 @@ use std::sync::atomic::Ordering::{AcqRel, Acquire, Relaxed};
 
 /// Scalable concurrent B+ tree.
 ///
-/// [`TreeIndex`] is a B+ tree variant that is optimized for read operations. Read operations,
-/// such as read, scan, are neither blocked nor interrupted by other threads. Write operations,
-/// such as insert, remove, do not block if they do not entail structural changes to the tree.
+/// [`TreeIndex`] is a B+ tree variant that is optimized for read operations. Read operations, such
+/// as read, scan, are neither blocked nor interrupted by other threads. Write operations, such as
+/// insert, remove, do not block if they do not entail structural changes to the tree. In case an
+/// operation is conflicted with another, one of them yields the current thread or task executor.
 ///
 /// ## The key features of [`TreeIndex`]
 ///
@@ -32,10 +32,9 @@ use std::sync::atomic::Ordering::{AcqRel, Acquire, Relaxed};
 ///
 /// ## The key statistics for [`TreeIndex`]
 ///
-/// * The maximum number of key-value pairs that a leaf can store: 8.
-/// * The maximum number of leaves or child nodes that a node can point to: 9.
-/// * The size of metadata per key-value pair in a leaf: 3-byte.
-/// * The size of metadata per leaf or node in a node: `size_of(K)` + 4.
+/// * The maximum number of key-value pairs that a leaf can store: 14.
+/// * The maximum number of leaves or child nodes that a node can point to: 15.
+/// * The size of metadata per key-value pair in a leaf: ~3-byte.
 pub struct TreeIndex<K, V>
 where
     K: 'static + Clone + Ord + Send + Sync,
@@ -57,8 +56,6 @@ where
     /// use scc::TreeIndex;
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
-    ///
-    /// assert!(treeindex.read(&1, |_, v| *v).is_none());
     /// ```
     #[must_use]
     pub fn new() -> TreeIndex<K, V> {
@@ -87,35 +84,114 @@ where
     #[inline]
     pub fn insert(&self, mut key: K, mut value: V) -> Result<(), (K, V)> {
         let barrier = Barrier::new();
-        let mut root_ptr = self.root.load(Acquire, &barrier);
         loop {
-            if let Some(root_ref) = root_ptr.as_ref() {
+            if let Some(root_ref) = self.root.load(Acquire, &barrier).as_ref() {
                 match root_ref.insert(key, value, &barrier) {
-                    Ok(_) => return Ok(()),
-                    Err(error) => match error {
-                        InsertError::Duplicated(entry) => return Err(entry),
-                        InsertError::Full(entry) => {
-                            root_ref.split_root(&self.root, &barrier);
-                            key = entry.0;
-                            value = entry.1;
-                        }
-                        InsertError::Retry(entry) => {
+                    Ok(r) => match r {
+                        InsertResult::Success => return Ok(()),
+                        InsertResult::Frozen(k, v) => {
+                            key = k;
+                            value = v;
                             std::thread::yield_now();
-                            key = entry.0;
-                            value = entry.1;
+                        }
+                        InsertResult::Duplicate(k, v) => return Err((k, v)),
+                        InsertResult::Full(k, v) => {
+                            let (k, v) = Node::split_root(k, v, &self.root, &barrier);
+                            key = k;
+                            value = v;
+                            continue;
+                        }
+                        InsertResult::Retired(k, v) => {
+                            key = k;
+                            value = v;
+                            if !matches!(Node::remove_root(&self.root, &barrier), Ok(true)) {
+                                std::thread::yield_now();
+                            }
                         }
                     },
+                    Err((k, v)) => {
+                        key = k;
+                        value = v;
+                        std::thread::yield_now();
+                    }
                 }
-                root_ptr = self.root.load(Acquire, &barrier);
-                continue;
             }
+
             let new_root = Arc::new(Node::new_leaf_node());
-            match self
-                .root
-                .compare_exchange(root_ptr, (Some(new_root), Tag::None), AcqRel, Acquire)
-            {
-                Ok((_, ptr)) | Err((_, ptr)) => root_ptr = ptr,
+            let _result = self.root.compare_exchange(
+                Ptr::null(),
+                (Some(new_root), Tag::None),
+                AcqRel,
+                Acquire,
+            );
+        }
+    }
+
+    /// Inserts a key-value pair.
+    ///
+    /// It is an asynchronous method returning an `impl Future` for the caller to await or poll.
+    ///
+    /// # Errors
+    ///
+    /// Returns an error along with the supplied key-value pair if the key exists.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::TreeIndex;
+    ///
+    /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
+    /// let future_insert = treeindex.insert_async(1, 10);
+    /// ```
+    #[inline]
+    pub async fn insert_async(&self, mut key: K, mut value: V) -> Result<(), (K, V)> {
+        loop {
+            let need_await = {
+                let barrier = Barrier::new();
+                if let Some(root_ref) = self.root.load(Acquire, &barrier).as_ref() {
+                    match root_ref.insert(key, value, &barrier) {
+                        Ok(r) => match r {
+                            InsertResult::Success => return Ok(()),
+                            InsertResult::Frozen(k, v) => {
+                                key = k;
+                                value = v;
+                                true
+                            }
+                            InsertResult::Duplicate(k, v) => return Err((k, v)),
+                            InsertResult::Full(k, v) => {
+                                let (k, v) = Node::split_root(k, v, &self.root, &barrier);
+                                key = k;
+                                value = v;
+                                continue;
+                            }
+                            InsertResult::Retired(k, v) => {
+                                key = k;
+                                value = v;
+                                !matches!(Node::remove_root(&self.root, &barrier), Ok(true))
+                            }
+                        },
+                        Err((k, v)) => {
+                            key = k;
+                            value = v;
+                            true
+                        }
+                    }
+                } else {
+                    false
+                }
+            };
+
+            if need_await {
+                async_yield::async_yield().await;
             }
+
+            let new_root = Arc::new(Node::new_leaf_node());
+            let _result = self.root.compare_exchange(
+                Ptr::null(),
+                (Some(new_root), Tag::None),
+                AcqRel,
+                Acquire,
+            );
         }
     }
 
@@ -141,6 +217,27 @@ where
         self.remove_if(key_ref, |_| true)
     }
 
+    /// Removes a key-value pair.
+    ///
+    /// It is an asynchronous method returning an `impl Future` for the caller to await or poll.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::TreeIndex;
+    ///
+    /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
+    /// let future_remove = treeindex.remove_async(&1);
+    /// ```
+    #[inline]
+    pub async fn remove_async<Q>(&self, key_ref: &Q) -> bool
+    where
+        K: Borrow<Q>,
+        Q: Ord + ?Sized,
+    {
+        self.remove_if_async(key_ref, |_| true).await
+    }
+
     /// Removes a key-value pair if the given condition is met.
     ///
     /// # Examples
@@ -160,46 +257,89 @@ where
         K: Borrow<Q>,
         Q: Ord + ?Sized,
     {
-        let mut has_been_removed = false;
-        let mut has_leaf_been_removed = false;
         let barrier = Barrier::new();
-        let mut root_ptr = self.root.load(Acquire, &barrier);
-        while let Some(root_ref) = root_ptr.as_ref() {
-            match root_ref.remove_if(key_ref, &mut condition, &barrier) {
-                Ok((removed, leaf_removed)) => {
-                    if leaf_removed || has_leaf_been_removed {
-                        if let Ok(leaf_scanner) = root_ref.max_less(key_ref, &barrier) {
-                            let _result = leaf_scanner.jump(None, &barrier);
+        let mut has_been_removed = false;
+        loop {
+            if let Some(root_ref) = self.root.load(Acquire, &barrier).as_ref() {
+                match root_ref.remove_if(key_ref, &mut condition, &barrier) {
+                    Ok(r) => match r {
+                        RemoveResult::Success => return true,
+                        RemoveResult::Fail => return has_been_removed,
+                        RemoveResult::Retired => {
+                            if matches!(Node::remove_root(&self.root, &barrier), Ok(true)) {
+                                return true;
+                            }
+                            has_been_removed = true;
+                        }
+                    },
+                    Err(removed) => {
+                        if removed {
+                            has_been_removed = true;
                         }
                     }
-                    return removed || has_been_removed;
                 }
-                Err(remove_error) => match remove_error {
-                    RemoveError::Empty((removed, leaf_removed)) => {
-                        if removed {
-                            has_been_removed = true;
+            } else {
+                return has_been_removed;
+            }
+            std::thread::yield_now();
+        }
+    }
+
+    /// Removes a key-value pair if the given condition is met.
+    ///
+    /// It is an asynchronous method returning an `impl Future` for the caller to await or poll.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use scc::TreeIndex;
+    ///
+    /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
+    /// let future_remove = treeindex.remove_if_async(&1, |v| *v == 0);
+    /// ```
+    #[inline]
+    pub async fn remove_if_async<Q, F: FnMut(&V) -> bool>(
+        &self,
+        key_ref: &Q,
+        mut condition: F,
+    ) -> bool
+    where
+        K: Borrow<Q>,
+        Q: Ord + ?Sized,
+    {
+        let mut has_been_removed = false;
+        loop {
+            let need_await = {
+                let barrier = Barrier::new();
+                if let Some(root_ref) = self.root.load(Acquire, &barrier).as_ref() {
+                    match root_ref.remove_if(key_ref, &mut condition, &barrier) {
+                        Ok(r) => match r {
+                            RemoveResult::Success => return true,
+                            RemoveResult::Fail => return has_been_removed,
+                            RemoveResult::Retired => {
+                                if matches!(Node::remove_root(&self.root, &barrier), Ok(true)) {
+                                    return true;
+                                }
+                                has_been_removed = true;
+                                true
+                            }
+                        },
+                        Err(removed) => {
+                            if removed {
+                                has_been_removed = true;
+                            }
+                            true
                         }
-                        if leaf_removed {
-                            has_leaf_been_removed = true;
-                        }
-                        if Node::remove_root(&self.root, &barrier) {
-                            return has_been_removed;
-                        }
-                    }
-                    RemoveError::Retry((removed, leaf_removed)) => {
-                        std::thread::yield_now();
-                        if removed {
-                            has_been_removed = true;
-                        }
-                        if leaf_removed {
-                            has_leaf_been_removed = true;
-                        }
                     }
-                },
+                } else {
+                    return has_been_removed;
+                }
             };
-            root_ptr = self.root.load(Acquire, &barrier);
+
+            if need_await {
+                async_yield::async_yield().await;
+            }
         }
-        has_been_removed
     }
 
     /// Reads a key-value pair.
@@ -212,10 +352,7 @@ where
     /// use scc::TreeIndex;
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
-    ///
     /// assert!(treeindex.read(&1, |k, v| *v).is_none());
-    /// assert!(treeindex.insert(1, 10).is_ok());
-    /// assert_eq!(treeindex.read(&1, |k, v| *v).unwrap(), 10);
     /// ```
     #[inline]
     pub fn read<Q, R, F: FnOnce(&Q, &V) -> R>(&self, key_ref: &Q, reader: F) -> Option<R>
@@ -240,11 +377,8 @@ where
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
     ///
-    /// assert!(treeindex.insert(1, 10).is_ok());
-    ///
     /// let barrier = Barrier::new();
-    /// let value_ref = treeindex.read_with(&1, |k, v| v, &barrier).unwrap();
-    /// assert_eq!(*value_ref, 10);
+    /// assert!(treeindex.read_with(&1, |k, v| v, &barrier).is_none());
     /// ```
     #[inline]
     pub fn read_with<'b, Q, R, F: FnOnce(&Q, &'b V) -> R>(
@@ -257,23 +391,9 @@ where
         K: Borrow<Q>,
         Q: Ord + ?Sized,
     {
-        let mut root_ptr = self.root.load(Acquire, barrier);
-        while let Some(root_ref) = root_ptr.as_ref() {
-            match root_ref.search(key_ref, barrier) {
-                Ok(result) => {
-                    if let Some(value) = result {
-                        return Some(reader(key_ref, value));
-                    }
-                    return None;
-                }
-                Err(err) => match err {
-                    SearchError::Empty => return None,
-                    SearchError::Retry => {
-                        std::thread::yield_now();
-                        root_ptr = self.root.load(Acquire, barrier);
-                        continue;
-                    }
-                },
+        if let Some(root_ref) = self.root.load(Acquire, barrier).as_ref() {
+            if let Some(value) = root_ref.search(key_ref, barrier) {
+                return Some(reader(key_ref, value));
             }
         }
         None
@@ -288,12 +408,7 @@ where
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
     ///
-    /// for key in 0..16_u64 {
-    ///     assert!(treeindex.insert(key, 10).is_ok());
-    /// }
-    ///
     /// treeindex.clear();
-    ///
     /// assert_eq!(treeindex.len(), 0);
     /// ```
     #[inline]
@@ -311,12 +426,7 @@ where
     /// use scc::TreeIndex;
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
-    ///
-    /// for key in 0..16_u64 {
-    ///     assert!(treeindex.insert(key, 10).is_ok());
-    /// }
-    ///
-    /// assert_eq!(treeindex.len(), 16);
+    /// assert_eq!(treeindex.len(), 0);
     /// ```
     #[inline]
     pub fn len(&self) -> usize {
@@ -350,13 +460,7 @@ where
     /// use scc::TreeIndex;
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
-    ///
-    /// for key in 0..16_u64 {
-    ///     let result = treeindex.insert(key, 10);
-    ///     assert!(result.is_ok());
-    /// }
-    ///
-    /// assert_eq!(treeindex.depth(), 1);
+    /// assert_eq!(treeindex.depth(), 0);
     /// ```
     #[inline]
     pub fn depth(&self) -> usize {
@@ -379,21 +483,13 @@ where
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
     ///
-    /// assert!(treeindex.insert(1, 10).is_ok());
-    /// assert!(treeindex.insert(2, 11).is_ok());
-    /// assert!(treeindex.insert(3, 13).is_ok());
-    ///
     /// let barrier = Barrier::new();
-    ///
     /// let mut visitor = treeindex.iter(&barrier);
-    /// assert_eq!(visitor.next().unwrap(), (&1, &10));
-    /// assert_eq!(visitor.next().unwrap(), (&2, &11));
-    /// assert_eq!(visitor.next().unwrap(), (&3, &13));
     /// assert!(visitor.next().is_none());
     /// ```
     #[inline]
     pub fn iter<'t, 'b>(&'t self, barrier: &'b Barrier) -> Visitor<'t, 'b, K, V> {
-        Visitor::new(self, barrier)
+        Visitor::new(&self.root, barrier)
     }
 
     /// Returns a [`Range`] that scans keys in the given range.
@@ -406,15 +502,8 @@ where
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
     ///
-    /// for i in 0..10 {
-    ///     assert!(treeindex.insert(i, 10).is_ok());
-    /// }
-    ///
     /// let barrier = Barrier::new();
-    ///
-    /// assert_eq!(treeindex.range(1..1, &barrier).count(), 0);
-    /// assert_eq!(treeindex.range(4..8, &barrier).count(), 4);
-    /// assert_eq!(treeindex.range(4..=8, &barrier).count(), 5);
+    /// assert_eq!(treeindex.range(4..=8, &barrier).count(), 0);
     /// ```
     #[inline]
     pub fn range<'t, 'b, R: RangeBounds<K>>(
@@ -422,39 +511,7 @@ where
         range: R,
         barrier: &'b Barrier,
     ) -> Range<'t, 'b, K, V, R> {
-        Range::new(self, range, barrier)
-    }
-}
-
-impl<K, V> TreeIndex<K, V>
-where
-    K: 'static + Clone + fmt::Display + Ord + Send + Sync,
-    V: 'static + Clone + fmt::Display + Send + Sync,
-{
-    /// Prints the [`TreeIndex`] contents to the given output in the DOT language.
-    ///
-    /// # Errors
-    ///
-    /// An [`io::Error`](std::io::Error) can be returned.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use scc::TreeIndex;
-    ///
-    /// let treeindex: TreeIndex<u64, u32> = TreeIndex::new();
-    ///
-    /// assert!(treeindex.insert(1, 10).is_ok());
-    ///
-    /// treeindex.print(&mut std::io::stdout());
-    /// ```
-    pub fn print<T: std::io::Write>(&self, output: &mut T) -> std::io::Result<()> {
-        output.write_fmt(format_args!("digraph {{\n"))?;
-        let barrier = Barrier::new();
-        if let Some(root_ref) = self.root.load(Acquire, &barrier).as_ref() {
-            root_ref.print(output, 1, &barrier)?;
-        }
-        output.write_fmt(format_args!("}}"))
+        Range::new(&self.root, range, barrier)
     }
 }
 
@@ -471,8 +528,6 @@ where
     /// use scc::TreeIndex;
     ///
     /// let treeindex: TreeIndex<u64, u32> = TreeIndex::default();
-    ///
-    /// assert!(treeindex.read(&1, |_, v| *v).is_none());
     /// ```
     fn default() -> Self {
         TreeIndex::new()
@@ -488,7 +543,7 @@ where
     K: 'static + Clone + Ord + Send + Sync,
     V: 'static + Clone + Send + Sync,
 {
-    tree: &'t TreeIndex<K, V>,
+    root: &'t AtomicArc<Node<K, V>>,
     leaf_scanner: Option<Scanner<'b, K, V>>,
     barrier: &'b Barrier,
 }
@@ -498,9 +553,12 @@ where
     K: 'static + Clone + Ord + Send + Sync,
     V: 'static + Clone + Send + Sync,
 {
-    fn new(tree: &'t TreeIndex<K, V>, barrier: &'b Barrier) -> Visitor<'t, 'b, K, V> {
+    pub(crate) fn new(
+        root: &'t AtomicArc<Node<K, V>>,
+        barrier: &'b Barrier,
+    ) -> Visitor<'t, 'b, K, V> {
         Visitor::<'t, 'b, K, V> {
-            tree,
+            root,
             leaf_scanner: None,
             barrier,
         }
@@ -516,16 +574,13 @@ where
     fn next(&mut self) -> Option<Self::Item> {
         // Starts scanning.
         if self.leaf_scanner.is_none() {
-            loop {
-                let root_ptr = self.tree.root.load(Acquire, self.barrier);
-                if let Some(root_ref) = root_ptr.as_ref() {
-                    if let Ok(scanner) = root_ref.min(self.barrier) {
-                        self.leaf_scanner.replace(scanner);
-                        break;
-                    }
-                } else {
-                    return None;
+            let root_ptr = self.root.load(Acquire, self.barrier);
+            if let Some(root_ref) = root_ptr.as_ref() {
+                if let Some(scanner) = root_ref.min(self.barrier) {
+                    self.leaf_scanner.replace(scanner);
                 }
+            } else {
+                return None;
             }
         }
 
@@ -565,7 +620,7 @@ where
     V: 'static + Clone + Send + Sync,
     R: 'static + RangeBounds<K>,
 {
-    tree: &'t TreeIndex<K, V>,
+    root: &'t AtomicArc<Node<K, V>>,
     leaf_scanner: Option<Scanner<'b, K, V>>,
     range: R,
     check_lower_bound: bool,
@@ -579,9 +634,13 @@ where
     V: 'static + Clone + Send + Sync,
     R: RangeBounds<K>,
 {
-    fn new(tree: &'t TreeIndex<K, V>, range: R, barrier: &'b Barrier) -> Range<'t, 'b, K, V, R> {
+    pub(crate) fn new(
+        root: &'t AtomicArc<Node<K, V>>,
+        range: R,
+        barrier: &'b Barrier,
+    ) -> Range<'t, 'b, K, V, R> {
         Range::<'t, 'b, K, V, R> {
-            tree,
+            root,
             leaf_scanner: None,
             range,
             check_lower_bound: true,
@@ -591,49 +650,61 @@ where
     }
 
     fn next_unbounded(&mut self) -> Option<(&'b K, &'b V)> {
-        // Starts scanning.
+        // Start scanning.
         if self.leaf_scanner.is_none() {
-            loop {
-                let root_ptr = self.tree.root.load(Acquire, self.barrier);
-                if let Some(root_ref) = root_ptr.as_ref() {
-                    let min_allowed_key = match self.range.start_bound() {
-                        Excluded(key) | Included(key) => Some(key),
-                        Unbounded => {
-                            self.check_lower_bound = false;
+            let root_ptr = self.root.load(Acquire, self.barrier);
+            if let Some(root_ref) = root_ptr.as_ref() {
+                let min_allowed_key = match self.range.start_bound() {
+                    Excluded(key) | Included(key) => Some(key),
+                    Unbounded => {
+                        self.check_lower_bound = false;
+                        None
+                    }
+                };
+                if let Some(leaf_scanner) = min_allowed_key.map_or_else(
+                    || {
+                        // Take the min entry.
+                        if let Some(mut min_scanner) = root_ref.min(self.barrier) {
+                            min_scanner.next();
+                            Some(min_scanner)
+                        } else {
                             None
                         }
+                    },
+                    |min_allowed_key| {
+                        // Take an entry that is close enough to the lower bound.
+                        root_ref.max_le_appr(min_allowed_key, self.barrier)
+                    },
+                ) {
+                    // Need to check the upper bound.
+                    self.check_upper_bound = match self.range.end_bound() {
+                        Excluded(key) => leaf_scanner
+                            .max_entry()
+                            .map_or(false, |max_entry| max_entry.0.cmp(key) != Ordering::Less),
+                        Included(key) => leaf_scanner
+                            .max_entry()
+                            .map_or(false, |max_entry| max_entry.0.cmp(key) == Ordering::Greater),
+                        Unbounded => false,
                     };
-                    if let Ok(leaf_scanner) = min_allowed_key.map_or_else(
-                        || root_ref.min(self.barrier),
-                        |min_allowed_key| root_ref.max_less(min_allowed_key, self.barrier),
-                    ) {
-                        self.check_upper_bound = match self.range.end_bound() {
-                            Excluded(key) => leaf_scanner
-                                .max_entry()
-                                .map_or(false, |max_entry| max_entry.0.cmp(key) != Ordering::Less),
-                            Included(key) => leaf_scanner.max_entry().map_or(false, |max_entry| {
-                                max_entry.0.cmp(key) == Ordering::Greater
-                            }),
-                            Unbounded => false,
-                        };
+                    if let Some(result) = leaf_scanner.get() {
                         self.leaf_scanner.replace(leaf_scanner);
-                        break;
+                        return Some(result);
                     }
-                } else {
-                    // Empty.
-                    return None;
                 }
+            } else {
+                // Empty.
+                return None;
             }
         }
 
-        // Proceeds to the next entry.
+        // Go to the next entry.
         if let Some(mut scanner) = self.leaf_scanner.take() {
             let min_allowed_key = scanner.get().map(|(key, _)| key);
             if let Some(result) = scanner.next() {
                 self.leaf_scanner.replace(scanner);
                 return Some(result);
             }
-            // Proceeds to the next leaf node.
+            // Go to the next leaf node.
             if let Some(new_scanner) = scanner.jump(min_allowed_key, self.barrier).take() {
                 if let Some(entry) = new_scanner.get() {
                     self.check_upper_bound = match self.range.end_bound() {
